﻿' DailyRemindersDialog.cs by Charles Petzold, March 2009

Imports Microsoft.VisualBasic
Imports System
Imports System.Windows
Imports System.Windows.Controls

Namespace DailyReminders
	Public Class DailyRemindersDialog
		Inherits Window
		Private storage As DailyRemindersStorage
		Private grid As Grid

		Public Sub New(ByVal owner As Window, ByVal storage As DailyRemindersStorage, ByVal dateTime As DateTime)
			Me.storage = storage

			ShowInTaskbar = False
			WindowStyle = WindowStyle.ToolWindow
			WindowStartupLocation = WindowStartupLocation.Manual
			Title = dateTime.ToLongDateString()
			Me.Owner = owner
			Width = 400
			Height = 500

			' Create the visual tree in code due to the repetition
			grid = New Grid()
			Dim coldef As New ColumnDefinition()
			coldef.Width = GridLength.Auto
			grid.ColumnDefinitions.Add(coldef)

			coldef = New ColumnDefinition()
			coldef.Width = New GridLength(1, GridUnitType.Star)
			grid.ColumnDefinitions.Add(coldef)

			' Prepare for the creation loop
			Dim yr As Integer = dateTime.Year
			Dim mn As Integer = dateTime.Month
			Dim dy As Integer = dateTime.Day
			Dim dtBeg As New DateTime(yr, mn, dy, 8, 0, 0)
			Dim dtEnd As New DateTime(yr, mn, dy, 23, 0, 0)
			Dim tsInc As TimeSpan = TimeSpan.FromMinutes(30)
			Dim row As Integer = 0

			Dim dt As DateTime = dtBeg
			Do While dt < dtEnd
				Dim rowdef As New RowDefinition()
				rowdef.Height = GridLength.Auto
				grid.RowDefinitions.Add(rowdef)

				Dim txtblk As New TextBlock()
				txtblk.Text = dt.ToShortTimeString()
				txtblk.Margin = New Thickness(6, 2, 6, 2)
				txtblk.HorizontalAlignment = HorizontalAlignment.Right
				txtblk.VerticalAlignment = VerticalAlignment.Center
				Grid.SetRow(txtblk, row)
				Grid.SetColumn(txtblk, 0)
				grid.Children.Add(txtblk)

				Dim txtbox As New TextBox()
				txtbox.AcceptsReturn = True
				txtbox.DataContext = dt
				txtbox.Text = storage.GetReminderText(dt)
				Grid.SetRow(txtbox, row)
				Grid.SetColumn(txtbox, 1)
				grid.Children.Add(txtbox)
				dt = dt.Add(tsInc)
				row += 1
			Loop
			Dim viewer As New ScrollViewer()
			viewer.VerticalScrollBarVisibility = ScrollBarVisibility.Visible
			viewer.Content = grid
			Content = viewer

			grid.Children(1).Focus()
		End Sub

		Protected Overrides Sub OnClosed(ByVal args As EventArgs)
			SaveAllReminders()
			MyBase.OnClosed(args)
		End Sub

		Public Sub SaveAllReminders()
			For Each child As UIElement In grid.Children
				If TypeOf child Is TextBox Then
					Dim txtbox As TextBox = TryCast(child, TextBox)
					storage.Update(CDate(txtbox.DataContext), txtbox.Text)
				End If
			Next child
			storage.Save()
		End Sub
	End Class
End Namespace
