﻿' DailyRemindersStorage.cs by Charles Petzold, March 2009

Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.IO
Imports System.IO.IsolatedStorage
Imports System.Xml

Namespace DailyReminders
	Public Class DailyRemindersStorage
		Private Const filename As String = "reminders.xml"
		Private reminders As New Dictionary(Of DateTime, String)()

		Public Sub New()
			Dim stream As IsolatedStorageFileStream

			Try
				stream = New IsolatedStorageFileStream(filename, FileMode.Open, FileAccess.Read)
			Catch
				Return
			End Try

			' Read XML 
			Dim reader As XmlReader = XmlReader.Create(stream)
			reader.ReadStartElement("Reminders")

			Do While reader.IsStartElement("Reminder")
				Dim str As String = reader.GetAttribute("DateTime")
				Dim dateTime As DateTime = DateTime.Parse(str)
				Dim text As String = reader.GetAttribute("Text")
				reminders.Add(dateTime, text)
				reader.ReadToNextSibling("Reminder")
			Loop
			reader.Close()
		End Sub

		Public Function GetReminderText(ByVal key As DateTime) As String
			Dim text As String

			If (Not reminders.TryGetValue(key, text)) Then
				text = ""
			End If

			Return text
		End Function

		Public Sub Update(ByVal key As DateTime, ByVal text As String)
			If text IsNot Nothing Then
				text = text.Trim()
			End If

			If String.IsNullOrEmpty(text) Then
				If reminders.ContainsKey(key) Then
					reminders.Remove(key)
				End If
			Else
				If reminders.ContainsKey(key) Then
					reminders(key) = text
				Else
					reminders.Add(key, text)
				End If
			End If
		End Sub

		Public Sub Save()
			Dim stream As New IsolatedStorageFileStream(filename, FileMode.Create, FileAccess.Write)

			' Write XML
			Dim settings As New XmlWriterSettings()
			settings.Indent = True
			Dim writer As XmlWriter = XmlWriter.Create(stream, settings)
			writer.WriteStartElement("Reminders")

			For Each key As DateTime In reminders.Keys
				writer.WriteStartElement("Reminder")
				writer.WriteAttributeString("DateTime", key.ToString("O"))
				writer.WriteAttributeString("Text", reminders(key))
				writer.WriteEndElement()
			Next key

			writer.WriteEndElement()
			writer.Close()
		End Sub
	End Class
End Namespace
