' MoonDisk.cs by Charles Petzold, March 2009

Imports Microsoft.VisualBasic
Imports System
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Media
Imports System.Windows.Media.Imaging
Imports System.Windows.Media.Media3D

Namespace MoonPhaseCalendar
	Partial Public Class MoonDisk
		Inherits Viewport3D
		' Create shareable model for moon.
		Private Shared strMoonImage As String = "pack://application:,,,/MoonPhaseCalendar;Component/moonmap.png"
		' (Original moonmap4k.jpg from from http://planetpixelemporium.com/planets.html, 
		'      resized to 25% and converted to PNG.) 
		Private Shared uriMoonImage As New Uri(strMoonImage)
		Private Shared imgMoon As New BitmapImage(uriMoonImage)
		Private Shared brush As New ImageBrush(imgMoon)
		Private Shared matMoon As New DiffuseMaterial(brush)
		Private Shared sphere As MeshGeometry3D = GenerateSphere(New Point3D(0, 0, 0), 1, 360, 180)
		Private Shared geomod As New GeometryModel3D(sphere, matMoon)

		' Create shareable model for lighting.
		Private Shared ambiLight As New AmbientLight(Color.FromRgb(16, 16, 16))
		Private Shared dirLight As New DirectionalLight(Colors.White, New Vector3D(0, 0, -1))
		Private Shared modgrp As New Model3DGroup()

		' Create shareable camera.
		Private Shared cam As New OrthographicCamera(New Point3D(0, 0, 2), New Vector3D(0, 0, -1), New Vector3D(0, 1, 0), 2.1)

		' Put together Model3DGroup for lighting & freeze the two models.
		Shared Sub New()
			geomod.Freeze()
			modgrp.Children.Add(ambiLight)

			For i As Integer = 0 To 3
				modgrp.Children.Add(dirLight)
			Next i

			modgrp.Freeze()
		End Sub

		Public Sub New()
			InitializeComponent()

			' Put together the Viewport3D.
			viewport3d.Camera = cam
			modvisGeometry.Content = geomod
			modvisLight.Content = modgrp

			' Set handler for DataContextChanged
			AddHandler DataContextChanged, AddressOf OnDataContextChanged
		End Sub

		Private Sub OnDataContextChanged(ByVal sender As Object, ByVal args As DependencyPropertyChangedEventArgs)
			rotate.Angle = PhaseAngle(CDate(args.NewValue))
		End Sub

		Private Shared Function GenerateSphere(ByVal center As Point3D, ByVal radius As Double, ByVal slices As Integer, ByVal stacks As Integer) As MeshGeometry3D
			' Create the MeshGeometry3D.
			Dim mesh As New MeshGeometry3D()

			' Fill the Position, Normals, and TextureCoordinates collections.
			For stack As Integer = 0 To stacks
				Dim phi As Double = Math.PI / 2 - stack * Math.PI / stacks
				Dim y As Double = radius * Math.Sin(phi)
				Dim scale As Double = -radius * Math.Cos(phi)

				For slice As Integer = 0 To slices
					Dim theta As Double = slice * 2 * Math.PI / slices
					Dim x As Double = scale * Math.Sin(theta)
					Dim z As Double = scale * Math.Cos(theta)

					Dim normal As New Vector3D(x, y, z)
					mesh.Normals.Add(normal)
					mesh.Positions.Add(normal + center)
					mesh.TextureCoordinates.Add(New Point(CDbl(slice) / slices, CDbl(stack) / stacks))
				Next slice
			Next stack

			' Fill the TriangleIndices collection.
			For stack As Integer = 0 To stacks - 1
				For slice As Integer = 0 To slices - 1
					Dim n As Integer = slices + 1 ' Keep the line length down.

					If stack <> 0 Then
						mesh.TriangleIndices.Add((stack + 0) * n + slice)
						mesh.TriangleIndices.Add((stack + 1) * n + slice)
						mesh.TriangleIndices.Add((stack + 0) * n + slice + 1)
					End If
					If stack <> stacks - 1 Then
						mesh.TriangleIndices.Add((stack + 0) * n + slice + 1)
						mesh.TriangleIndices.Add((stack + 1) * n + slice)
						mesh.TriangleIndices.Add((stack + 1) * n + slice + 1)
					End If
				Next slice
			Next stack
			Return mesh
		End Function

		' All formula references are from 
		'  Jean Meeus, Astronomical Algorithms, first edition (Willmann-Bell, 1991)
		Public Function PhaseAngle(ByVal dtInput As DateTime) As Double
			Dim dt As DateTime = dtInput

			If dtInput.Kind = DateTimeKind.Local Then
				dt = dtInput.ToUniversalTime()
			End If

			' 7.1: Julian Day
			' ---------------
			Dim yr As Integer = dt.Year
			Dim mon As Integer = dt.Month
			Dim day As Double = dt.Day + (dt.Hour + (dt.Minute + (dt.Second + dt.Millisecond / 1000.0) / 60.0) / 60.0) / 24.0

			If mon = 1 OrElse mon = 2 Then
				yr -= 1
				mon += 12
			End If

			Dim A As Integer = yr \ 100
			Dim B As Integer = 2 - A + A \ 4

			Dim JD As Double = Math.Floor(365.25 * (yr + 4716)) + Math.Floor(30.6001 * (mon + 1)) + day + B - 1524.5

			' Julian Emphemeris Day (approximate -- see chapter 9)
			' ----------------------------------------------------
			Dim JDE As Double = JD

			' 21.1: Time measured in Julian centures from Epoch J2000.0
			' ---------------------------------------------------------
			Dim tau As Double = (JDE - 2451545) / 36525

			' 45.2: Mean elongation of moon
			' -----------------------------
			Dim D As Double = 297.8501921 + 445267.1114034 * tau - 0.0018819 * tau * tau + tau * tau * tau / 545868 - tau * tau * tau * tau / 113065000

			D = Normalize(D)

			' 45.3: Sun's mean anomaly
			' ------------------------
			Dim M As Double = 357.5291092 + 35999.0502909 * tau - 0.0001536 * tau * tau + tau * tau * tau / 24490000

			M = Normalize(M)

			' 45.4: Moon's mean anomaly
			' -------------------------
			Dim Mprime As Double = 134.9633964 + 477198.8675055 * tau + 0.0087414 * tau * tau + tau * tau * tau / 69699 - tau * tau * tau * tau / 14712000

			Mprime = Normalize(Mprime)

			' 46.4: Phase angle (simplified calculation)
			' ------------------------------------------
			Dim i As Double = 180 - D - 6.289 * Sine(Mprime) + 2.100 * Sine(M) - 1.274 * Sine(2 * D - Mprime) - 0.658 * Sine(2 * D) - 0.214 * Sine(2 * Mprime) - 0.110 * Sine(D)

			' 46.1: Illuminated fraction (not used)
			' --------------------------
			Dim k As Double = (1 + Cosine(i)) / 2

			Return i

		End Function

		Private Function Normalize(ByVal angle As Double) As Double
			angle = angle - 360 * CInt(Fix(angle / 360))

			If angle < 0 Then
				angle += 360
			End If

			Return angle
		End Function
		Private Function Sine(ByVal angle As Double) As Double
			Return Math.Sin(Math.PI * angle / 180)
		End Function
		Private Function Cosine(ByVal angle As Double) As Double
			Return Math.Cos(Math.PI * angle / 180)
		End Function
	End Class
End Namespace
